/*
  PC Status Monitor Daemon for LCD2USB

  T. Nakagawa
  2013/12/28
*/


#include <netinet/in.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <syslog.h>
#include <unistd.h>
#include "clcd.h"


#define IDENT "lcdmonitord"
#define LCD_CONTRAST 255
#define LCD_BRIGHTNESS 127
#define SLEEP 3	 /* Update interval [sec] */
#define CORETEMP_FILE "/sys/devices/platform/coretemp.0/temp3_input"
#define HDDTEMP_PORT 7634
#define HDDTEMP_ADDR ((127 << 24) | (0 << 16) | (0 << 8) | 1)  /* localhost */


void sighandler(int sig) {
  clcd_print("lcdmonitord was stopped.        ");
  clcd_close();
  syslog(LOG_INFO, "Terminated");
  exit(0);
}


double get_cpuload(void) {
  FILE *fp;
  double load = 0.0;

  if ((fp = fopen("/proc/loadavg", "r")) != NULL) {
    fscanf(fp, "%lf", &load);
    fclose(fp);
  }

  return load;
}


int get_freemem(void) {
  FILE *fp;
  int mem = 0;

  if ((fp = fopen("/proc/meminfo", "r")) != NULL) {
    fscanf(fp, "%*s %*d %*s MemFree: %d kB", &mem);	/* Free memory in kB */
    fclose(fp);
  }

  return mem;
}


double get_cputemp(void) {
  FILE *fp;
  double temp = 0.0;

  if ((fp = fopen(CORETEMP_FILE, "r")) != NULL) {
    fscanf(fp, "%lf", &temp);
    fclose(fp);
    temp /= 1000.0;
  }

  return temp;
}


double get_hddtemp(void) {
  int fd;
  double temp = 0.0;

  if ((fd = socket(PF_INET, SOCK_STREAM, 0)) >= 0) {
    struct sockaddr_in addr;
    char buf[256];
    char *ptr;
    int len = 0;

    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_port = htons(HDDTEMP_PORT);
    addr.sin_addr.s_addr = htonl(HDDTEMP_ADDR);
    if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)) >= 0) {
      for (; ; ) {
	int size;
	size = read(fd, buf + len, sizeof(buf) - len - 1);
	if (size <= 0) break;
	len += size;
      }
    }
    close(fd);
    if (len > 0) {
      buf[len] = '\0';
      if ((ptr = strchr(buf, '|')) != NULL &&
	  (ptr = strchr(ptr + 1, '|')) != NULL &&
	  (ptr = strchr(ptr + 1, '|')) != NULL) {
	temp = atof(ptr + 1);
      }
    }
  }

  return temp;
}


int main(void) {
  struct sigaction sa;

  openlog(IDENT, (LOG_CONS | LOG_PERROR | LOG_PID), LOG_DAEMON);

  if (clcd_open()) {
    syslog(LOG_ERR, "clcd_open() failed: %s", clcd_error);
    return 2;
  }
  clcd_setcontrast(LCD_CONTRAST);
  clcd_setbrightness(LCD_BRIGHTNESS);
  clcd_print("lcdmonitord was started.        ");
  sleep(1);

  sa.sa_handler = sighandler;
  sigemptyset(&sa.sa_mask);
  if (sigaction(SIGTERM, &sa, NULL) < 0) {
    syslog(LOG_ERR, "sigaction() failed");
    return 4;
  }

  if (daemon(0, 0)) {
    syslog(LOG_ERR, "daemon() failed");
    return 3;
  }

  syslog(LOG_INFO, "Started");

  for (; ; ) {
    char buf[256];
    double cpuload;
    int freemem;
    double cputemp;
    double hddtemp;

    cpuload = get_cpuload();
    freemem = get_freemem();
    cputemp = get_cputemp();
    hddtemp = get_hddtemp();

    sprintf(buf, "Load:%4.2f CPU:%2dMem:%4dM HDD:%2d", cpuload, (int)(cputemp + 0.5), (int)(freemem / 1024 + 0.5), (int)(hddtemp + 0.5));
    clcd_print(buf);

    sleep(SLEEP);
  }

  return 0;
}
